/*
 * IV-27 SPI Display test
 * 
 * Code by Manuel Azevedo (azevedo.manuel@gmail.com)
 * Based on Pascal code by kama-labs.com
 */

#include <SPI.h>

/*
 * Test with Arduino UNO: 
 *  STRO      (Blank)  9
 *  CS        (Load)  10
 *  SPI MOSI  (DIN)   11
 *  SPI MISO  (DOUT)  12 
 *  SPI CLOCK (CLK)   13
 *  
 */

const byte STRO = 9;
const byte LOAD = 10;  

word textOutLo;        // Last word in
word textOutHi;        // First word in

byte dinDigit = 0;     // Digit to show


/*
 * Segment bit table 
 * Documentation segment leter assignment is wrong. 
 * This is the correct order 
 * 
Digit h d b e g f c a 
     ----------------------
  0 | 0 0 1 1 1 1 1 1 |  63
  1 | 0 0 0 0 0 1 1 0 |   6
  2 | 0 1 0 1 1 0 1 1 |  91
  3 | 0 1 0 0 1 1 1 1 |  79
  4 | 0 1 1 0 0 1 1 0 | 102
  5 | 0 1 1 0 1 1 0 1 | 109
  6 | 0 1 1 1 1 1 0 1 | 125
  7 | 0 0 0 0 0 1 1 1 |   7
  8 | 0 1 1 1 1 1 1 1 | 127
  9 | 0 1 1 0 1 1 1 1 | 111
  = | 0 1 0 0 1 0 0 1 |  73
  A | 0 1 1 1 0 1 1 1 | 119
  C | 0 0 1 1 1 0 0 1 |  57
  ! | 0 1 0 1 0 0 0 1 |  81
    ------------------------
 */
                     // 0    1   2    3    4    5    6    7    8    9    =    A    C    !
word digits[14]      = {63  ,6  ,91  ,79  ,102 ,109 ,125 ,7   ,127 ,111 ,73  ,119 ,57  ,81  };
word masDinDigit[14] = {1   ,2  ,4   ,8   ,16  ,32  ,64  ,128 ,256 ,512 ,1024,2048,4096,8192};
                     // G1   G2  G3   G4   G5   G6   G7   G8   G9   G10  G11  G12  G13  G14 

// Settings for UNO
// HV518 is a 6Mhz device, Most Significate Bit First, SPI Mode 3 
SPISettings settings(6000000,MSBFIRST, SPI_MODE3);

// Cycle through digits[] and show each digit at position masDinDigit[]
void timerOut(void){

  if (dinDigit > 13) 
    dinDigit=0;

  // Digits are most significant, so go first (Hi)
  textOutLo=masDinDigit[dinDigit];
  textOutHi=digits[dinDigit];
  
  SPI.beginTransaction(settings);
  SPI.transfer16(textOutHi);
  SPI.transfer16(textOutLo);
  SPI.endTransaction();

  //LOAD LOW to show stored registers
  digitalWrite(LOAD, LOW); 

  // LOAD HIGH to make HV518 receive new digits again
  digitalWrite(LOAD, HIGH);

  // Next digit
  dinDigit++;
 
}


void setup() {

  // Define LOAD and STRO pins as OUTPUT
  pinMode(LOAD, OUTPUT);
  pinMode(STRO, OUTPUT);
  // LOAD HIGH (receive data)
  digitalWrite(LOAD, HIGH); 
  // STRO LOW - HV outputs are Enabled
  digitalWrite(STRO, LOW); 
  
  SPI.begin();

  // Clear the display of any garbage
  SPI.beginTransaction(settings);
  SPI.transfer16(word(0,0));
  SPI.endTransaction();

}

void loop() {
  timerOut();
  delay(500);
}
